/*
 * Copyright 2008 the original author or authors.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package de.timefinder.core;

import de.timefinder.algo.DataPoolSettings;
import de.timefinder.algo.io.DataPool4Track2;
import de.timefinder.core.io.ImportText;
import de.timefinder.core.io.xml.ImportXmlXStream;
import de.timefinder.core.io.xml.ImportXmlXvantage;
import de.timefinder.core.io.xml.bt.ImportBTXml;
import de.timefinder.core.util.ApplicationSettings;
import de.timefinder.core.util.Translator;
import de.timefinder.data.DataPool;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GraphicsEnvironment;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.Map.Entry;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.richclient.application.ApplicationWindow;
import org.springframework.richclient.application.config.ApplicationWindowConfigurer;
import org.springframework.richclient.application.config.DefaultApplicationLifecycleAdvisor;
import org.springframework.richclient.application.setup.SetupWizard;

/**
 * Custom application lifecycle implementation that configures the application
 * 
 * @author Peter Karich
 */
public class LifecycleAdvisor extends DefaultApplicationLifecycleAdvisor {

    private final Log logger = LogFactory.getLog(getClass());
    private ApplicationSettings appSettings;
    private DataPoolSettings dataSettings;
    private Translator tr;
    private DataPool dataPool;

    @Override
    public void onPreStartup() {
        if (appSettings.getShowStartupWizard() &&
                getApplication().getApplicationContext().containsBean("setupWizard")) {
            SetupWizard setupWizard = (SetupWizard) getApplication().
                    getApplicationContext().getBean("setupWizard", SetupWizard.class);

            setupWizard.execute();

            appSettings.setShowStartupWizard(false);
            try {
                appSettings.save();
            } catch (IOException ex) {
                logger.error("Can't save state.", ex);
            }
        }
    }

    public void setSettings(ApplicationSettings set) {
        appSettings = set;
    }

    @Autowired
    public void setDataPoolSettings(DataPoolSettings settings) {
        dataSettings = settings;
    }

    @Autowired
    public void setTranslator(Translator tr) {
        this.tr = tr;
    }

    @Autowired
    public void setDataPool(DataPool dataPool) {
        this.dataPool = dataPool;
    }

    /**
     * This method is called prior to the opening of an application window. Note
     * at this point the window control has not been created. This hook allows
     * programmatic control over the configuration of the window (by setting
     * properties on the configurer) and it provides a hook where code that
     * needs to be executed prior to the window opening can be plugged in (like
     * a startup wizard, for example).
     * 
     * @param configurer The application window configurer
     */
    @Override
    public void onPreWindowOpen(ApplicationWindowConfigurer configurer) {

        // If you override this method, it is critical to allow the superclass
        // implementation to run as well.
        super.onPreWindowOpen(configurer);

        // Uncomment to hide the menubar, toolbar, or alter window size...
        // configurer.setShowMenuBar(false);
        // configurer.setShowToolBar(false);
        Rectangle rect = GraphicsEnvironment.getLocalGraphicsEnvironment().getMaximumWindowBounds();
        configurer.setInitialSize(new Dimension(rect.width, rect.height));
    }

    /**
     * Called just after the command context has been internalized. At this
     * point, all the commands for the window have been created and are
     * available for use. If you need to force the execution of a command prior
     * to the display of an application window (like a login command), this is
     * where you'd do it.
     * 
     * @param window The window who's commands have just been created
     */
    @Override
    public void onCommandsCreated(ApplicationWindow window) {
    }

    /**
     * Called after the actual window control has been created.
     * 
     * @param window The window being processed
     */
    @Override
    public void onWindowCreated(ApplicationWindow window) {
    }

    /**
     * Called immediately after making the window visible.
     * 
     * @param window The window being processed
     */
    @Override
    public void onWindowOpened(ApplicationWindow window) {
    }

    /**
     * Called when the window is being closed. This hook allows control over
     * whether the window is allowed to close. By returning false from this
     * method, the window will not be closed.
     *
     * @return boolean indicator if window should be closed. <code>true</code>
     *         to allow the close, <code>false</code> to prevent the close.
     */
    @Override
    public boolean onPreWindowClose(ApplicationWindow window) {
        if (appSettings.getShowExitQuestion()) {
            JPanel messageComponent = new JPanel(new BorderLayout());
            messageComponent.add(new JLabel(tr.get("exitCommand.confirm.message")),
                    BorderLayout.CENTER);
            final JCheckBox box = new JCheckBox(tr.get("exitCommand.showAgain"), true);
            box.setForeground(Color.GRAY);
            box.addActionListener(new ActionListener() {

                @Override
                public void actionPerformed(ActionEvent e) {
                    if (!box.isSelected()) {
                        appSettings.setShowExitQuestion(false);
                        try {
                            appSettings.save();
                        } catch (IOException ex) {
                            logger.warn("Couldn't save exit-message-show-state", ex);
                        }
                    }
                }
            });
            messageComponent.add(box, BorderLayout.SOUTH);
            return JOptionPane.YES_OPTION ==
                    JOptionPane.showConfirmDialog(window.getControl(),
                    messageComponent, tr.get("exitCommand.confirm.title"),
                    JOptionPane.YES_NO_OPTION);
        } else {
            return true;
        }
    }

    /**
     * Called when the application has fully started. This is after the initial
     * application window has been made visible.
     */
    @Override
    public void onPostStartup() {
        Entry<Class, File> entry = appSettings.getLastImport();

        if (entry != null) {

            String[] options = new String[]{tr.get("yesCommand.caption"),
                tr.get("noCommand.caption"),
                tr.get("autoImport.alwaysSkip")};
            int res = JOptionPane.showOptionDialog(null,
                    tr.get("autoImport.question", entry.getValue()),
                    tr.get("autoImport"), JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE,
                    null, options, tr.get("yesCommand.caption"));
            if (res == 1 || res == JOptionPane.CLOSED_OPTION)
                return;

            if (res == 2) {
                appSettings.setLastImport(null);
                try {
                    appSettings.save();
                } catch (IOException ex) {
                    logger.fatal("Auto-Import message will pop up next time again :-(");
                }
                return;
            }

            try {
                if (entry.getKey().isAssignableFrom(ImportXmlXvantage.class)) {
                    new ImportXmlXvantage(dataPool, dataSettings, new FileReader(entry.getValue())).doWork();
                } else if (entry.getKey().isAssignableFrom(ImportXmlXStream.class)) {
                    new ImportXmlXStream(dataPool, dataSettings, new FileReader(entry.getValue())).doWork();
                } else if (entry.getKey().isAssignableFrom(ImportText.class)) {
                    new ImportText(dataPool, dataSettings, entry.getValue()).doWork();
                } else if (entry.getKey().isAssignableFrom(ImportBTXml.class)) {
                    ImportBTXml importer = new ImportBTXml(dataPool, entry.getValue());
                    importer.doWork();
                } else if (entry.getKey().isAssignableFrom(DataPool4Track2.class)) {

                    DataPool4Track2 track2 = new DataPool4Track2();
                    track2.setDataPool(dataPool);
                    track2.setSettings(dataSettings);
                    track2.changeSettings();
                    track2.setSource(entry.getValue());
                    track2.doWork();
                }
            } catch (Exception ex) {
                // TODO clear data?

                logger.warn("Couldn't import!", ex);
                getApplication().getActiveWindow().getStatusBar().setErrorMessage("Couldn't import from " +
                        entry.getValue() + ". " + ex.getLocalizedMessage());
            }
        }
    }
}
