/*
 *  Copyright 2009 Peter Karich, peat_hal ‘at’ users ‘dot’ sourceforge ‘dot’ net.
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *  under the License.
 */
package de.timefinder.core.io.xml;

import de.timefinder.algo.DataPoolSettings;
import de.timefinder.algo.constraint.RasterConstraint;
import de.timefinder.core.util.FixRelationShip;
import de.timefinder.data.DataPool;
import de.timefinder.data.DBInterface;
import de.timefinder.data.Value;
import de.timefinder.data.access.Dao;
import java.io.IOException;
import java.io.InputStream;
import java.io.Reader;
import java.util.List;
import org.springframework.richclient.application.statusbar.StatusBar;

/**
 * We need to set one relationship of the unidirectional relations
 * (e.g. person<->event) because we removed this in the export method
 * to avoid ugly referencing (a very deeply nested one) in the xml.
 *
 * @author Peter Karich, peat_hal ‘at’ users ‘dot’ sourceforge ‘dot’ net
 */
public class ImportXmlXStream extends AbstractXStreamHelper {

    private DataPoolSettings poolSettings;
    private StatusBar statusBar;

    public ImportXmlXStream(DataPool pool, DataPoolSettings settings, final Reader reader) {
        super(pool);
        init(settings, reader);
    }

    public ImportXmlXStream(DataPool pool, DataPoolSettings settings, final InputStream is) {
        super(pool);
        init(settings, is);
    }

    protected void init(DataPoolSettings settings, final Object reader) {
        poolSettings = settings;
        setXStreamReader(reader);
    }

    public void setStatusBar(StatusBar bar) {
        statusBar = bar;
    }

    public void updateStatus(int percentage) {
        if (statusBar != null)
            statusBar.getProgressMonitor().worked(percentage);
    }

    @SuppressWarnings("unchecked")
    private void fillDaos() {
        // It is important to note that we need to re-set bidirectional relations
        // because we write only unidirectional relationships (to avoid ugly xml with xstream)
        // pros: * simpler xml, * could be read and write with xstream, * no duplicate referencing necessary
        // cons: * additional setup while writing and reading (but again: we can use xstream )
        //        1. person
        //        2. feature
        //        3. location
        //        4. event
        updateStatus(50);
        double tmp = 30.0 / (listsOfDbObjectList.size() + 1);
        int counter = 0;

        log.info("set:" + poolSettings.getTimeslotsPerDay() + " days:" + poolSettings.getNumberOfDays());
        for (MyArrayList<? extends DBInterface> objList : listsOfDbObjectList) {
            updateStatus(50 + (int) (counter * tmp));
            counter++;
            Dao<? extends DBInterface> dao = dataPool.getDao(objList.getType());

            if (dao == null) {
                log.error("Type " + objList.getType() + " is unsupported");
                continue;
            }
            dao.detachAll();
            ((Dao<DBInterface>) dao).attachAll(objList.getList());

            // if no raster was found initialize with empty rasters.
            // TODO only resources (person, location, feature) and events
            for (DBInterface obj : dao.getAll()) {
                Value val = ((Value) obj);
                if (val.getConstraint(RasterConstraint.class) == null) {
                    val.putConstraint(new RasterConstraint(poolSettings.createWeekRaster()));
                }
            }
        }

        updateStatus(90);

        new FixRelationShip(dataPool).doWork();
    }

    private void setXStreamReader(final Object reader) {
        xStreamReader = new XStreamReadFromXml() {

            @Override
            @SuppressWarnings("unchecked")
            public void readXml() throws IOException {
                Object[] objects;
                if (reader instanceof Reader)
                    objects = (Object[]) xstream.fromXML((Reader) reader);
                else
                    objects = (Object[]) xstream.fromXML((InputStream) reader);
                poolSettings.apply((DataPoolSettings) objects[0]);
                listsOfDbObjectList = (List<MyArrayList<? extends DBInterface>>) objects[1];

                if (reader instanceof Reader)
                    ((Reader) reader).close();
                else
                    ((InputStream) reader).close();

                fillDaos();
            }
        };
    }
}
